@@ -54,8 +54,11 @@ class Statistics:
         return self.num_failures / self.num_samples
 
     def variance(self):
-        return self.success_rate() * self.failure_rate() / self.num_samples 
-        
+        return (
+            ( self.num_successes * self.success_rate() 
+            + self.num_failures * self.failure_rate()
+            ) / (self.num_experiments - 1)
+        )
 
     def std(self):
         return math.sqrt(self.variance())
@@ -64,7 +67,7 @@ class Statistics:
         string = "Statistics\n"
         string += "----------\n"
         string += f"number of samples: {self.num_samples}\n"
-        if self.num_samples > 0:
+        if self.sample_size() > 0:
             string += f"failure rate: {self.failure_rate()}\n"
             string += f"success rate: {self.success_rate()}\n"
             string += f"standard deviation: {self.std()}"
@@ -78,10 +81,14 @@ class LinearDecodingExperiment:
         self.noise = noise
 
     def run_once(self, rng):
-        error = self.noise.sample(len(self.code), rng) 
-        guess = self.decoder.decode(error)
-        return guess.is_zero()
-   
+        codeword = self.code.random_codeword(rng)
+        error = self.noise.sample(len(self.code), rng)
+        guess = self.decoder.decoder(codeword + error)
+        if guess is not None:
+            return codeword == guess  
+        else:
+            return False
+
     def run_while(self, condition, rng):
         statistics = Statistics()
         while condition(statistics):
@@ -102,12 +109,12 @@ class LinearDecodingExperiment:
 
 def convert_to_dataframe(experiment, statistics, extras=dict()):
     data = {
-        "Code length": [len(experiment.code)],
-        "Code dimension": [experiment.code.dimension()],
-        "Physical error rate": [experiment.noise.error_probability()],
-        "Number of samples": [statistics.num_samples],
-        "Logical failure rate": [statistics.failure_rate()],
-        "Standard deviation": [statistics.std()],
+        "Block length": len(experiment.code),
+        "Dimension": experiment.code.dimension(),
+        "Physical error rate": experiment.noise.error_probability(),
+        "Number of experiments": statistics.num_experiments(),
+        "Logical failure rate": statistics.failure_rate(),
+        "Standard deviation": statistics.std(),
     }
     data.update(extras)
-    return DataFrame.from_dict(data)
+    return DataFrame(data)
